Stan Math Library: stan/math/prim/scal/prob/scaled_inv_chi_square_ccdf_log.hpp Source File

| Stan Math Library 2.10.0 reverse mode automatic differentiation

- Main Page
- Modules
- Namespaces
- Classes
- Files

- File List
- File Members

scaled_inv_chi_square_ccdf_log.hpp
Go to the documentation of this file.
1 #ifndef STAN_MATH_PRIM_SCAL_PROB_SCALED_INV_CHI_SQUARE_CCDF_LOG_HPP
2 #define STAN_MATH_PRIM_SCAL_PROB_SCALED_INV_CHI_SQUARE_CCDF_LOG_HPP
3
4 #include <stan/math/prim/scal/meta/is_constant_struct.hpp>
5 #include <stan/math/prim/scal/meta/partials_return_type.hpp>
6 #include <stan/math/prim/scal/meta/OperandsAndPartials.hpp>
7 #include <stan/math/prim/scal/err/check_consistent_sizes.hpp>
8 #include <stan/math/prim/scal/err/check_nonnegative.hpp>
9 #include <stan/math/prim/scal/err/check_not_nan.hpp>
10 #include <stan/math/prim/scal/err/check_positive_finite.hpp>
11 #include <stan/math/prim/scal/fun/constants.hpp>
12 #include <stan/math/prim/scal/fun/value_of.hpp>
13 #include <stan/math/prim/scal/fun/gamma_q.hpp>
14 #include <stan/math/prim/scal/fun/digamma.hpp>
15 #include <stan/math/prim/scal/fun/lgamma.hpp>
16 #include <stan/math/prim/scal/fun/square.hpp>
17 #include <stan/math/prim/scal/meta/VectorView.hpp>
18 #include <stan/math/prim/scal/meta/VectorBuilder.hpp>
19 #include <stan/math/prim/scal/meta/length.hpp>
20 #include <stan/math/prim/scal/meta/include_summand.hpp>
21 #include <stan/math/prim/scal/fun/grad_reg_inc_gamma.hpp>
22 #include <boost/random/chi_squared_distribution.hpp>
23 #include <boost/random/variate_generator.hpp>
24 #include <limits>
25 #include <cmath>
26
27 namespace stan {
28
29 namespace math {
30
31 template <typename T_y, typename T_dof, typename T_scale>
32 typename return_type<T_y, T_dof, T_scale>::type
33 scaled_inv_chi_square_ccdf_log(const T_y& y, const T_dof& nu,
34 const T_scale& s) {
35 typedef typename stan::partials_return_type<T_y, T_dof, T_scale>::type
36 T_partials_return;
37
38 // Size checks
39 if (!(stan::length(y) && stan::length(nu) && stan::length(s)))
40 return 0.0;
41
42 static const char* function("stan::math::scaled_inv_chi_square_ccdf_log");
43
44 using stan::math::check_positive_finite;
45 using stan::math::check_not_nan;
46 using stan::math::check_consistent_sizes;
47 using stan::math::check_nonnegative;
48 using stan::math::value_of;
49 using std::exp;
50
51 T_partials_return P(0.0);
52
53 check_not_nan(function, "Random variable", y);
54 check_nonnegative(function, "Random variable", y);
55 check_positive_finite(function, "Degrees of freedom parameter", nu);
56 check_positive_finite(function, "Scale parameter", s);
57 check_consistent_sizes(function,
58 "Random variable", y,
59 "Degrees of freedom parameter", nu,
60 "Scale parameter", s);
61
62 // Wrap arguments in vectors
63 VectorView<const T_y> y_vec(y);
64 VectorView<const T_dof> nu_vec(nu);
65 VectorView<const T_scale> s_vec(s);
66 size_t N = max_size(y, nu, s);
67
68 OperandsAndPartials<T_y, T_dof, T_scale>
69 operands_and_partials(y, nu, s);
70
71 // Explicit return for extreme values
72 // The gradients are technically ill-defined, but treated as zero
73 for (size_t i = 0; i < stan::length(y); i++) {
74 if (value_of(y_vec[i]) == 0)
75 return operands_and_partials.value(0.0);
76 }
77
78 // Compute cdf_log and its gradients
79 using stan::math::gamma_q;
80 using stan::math::digamma;
81 using boost::math::tgamma;
82 using std::exp;
83 using std::pow;
84 using std::log;
85
86 // Cache a few expensive function calls if nu is a parameter
87 VectorBuilder<!is_constant_struct<T_dof>::value,
88 T_partials_return, T_dof> gamma_vec(stan::length(nu));
89 VectorBuilder<!is_constant_struct<T_dof>::value,
90 T_partials_return, T_dof> digamma_vec(stan::length(nu));
91
92 if (!is_constant_struct<T_dof>::value) {
93 for (size_t i = 0; i < stan::length(nu); i++) {
94 const T_partials_return half_nu_dbl = 0.5 * value_of(nu_vec[i]);
95 gamma_vec[i] = tgamma(half_nu_dbl);
96 digamma_vec[i] = digamma(half_nu_dbl);
97 }
98 }
99
100 // Compute vectorized cdf_log and gradient
101 for (size_t n = 0; n < N; n++) {
102 // Explicit results for extreme values
103 // The gradients are technically ill-defined, but treated as zero
104 if (value_of(y_vec[n]) == std::numeric_limits<double>::infinity()) {
105 return operands_and_partials.value(stan::math::negative_infinity());
106 }
107
108 // Pull out values
109 const T_partials_return y_dbl = value_of(y_vec[n]);
110 const T_partials_return y_inv_dbl = 1.0 / y_dbl;
111 const T_partials_return half_nu_dbl = 0.5 * value_of(nu_vec[n]);
112 const T_partials_return s_dbl = value_of(s_vec[n]);
113 const T_partials_return half_s2_overx_dbl = 0.5 * s_dbl * s_dbl
114 * y_inv_dbl;
115 const T_partials_return half_nu_s2_overx_dbl
116 = 2.0 * half_nu_dbl * half_s2_overx_dbl;
117
118 // Compute
119 const T_partials_return Pn = 1.0 - gamma_q(half_nu_dbl,
120 half_nu_s2_overx_dbl);
121 const T_partials_return gamma_p_deriv = exp(-half_nu_s2_overx_dbl)
122 * pow(half_nu_s2_overx_dbl, half_nu_dbl-1) / tgamma(half_nu_dbl);
123
124 P += log(Pn);
125
126 if (!is_constant_struct<T_y>::value)
127 operands_and_partials.d_x1[n] -= half_nu_s2_overx_dbl * y_inv_dbl
128 * gamma_p_deriv / Pn;
129 if (!is_constant_struct<T_dof>::value)
130 operands_and_partials.d_x2[n]
131 -= (0.5 * stan::math::grad_reg_inc_gamma(half_nu_dbl,
132 half_nu_s2_overx_dbl,
133 gamma_vec[n],
134 digamma_vec[n])
135 - half_s2_overx_dbl * gamma_p_deriv)
136 / Pn;
137 if (!is_constant_struct<T_scale>::value)
138 operands_and_partials.d_x3[n] += 2.0 * half_nu_dbl * s_dbl * y_inv_dbl
139 * gamma_p_deriv / Pn;
140 }
141
142 return operands_and_partials.value(P);
143 }
144 }
145 }
146 #endif
check_not_nan.hpp
stan::math::OperandsAndPartials::d_x2
VectorView< T_return_type, false, true > d_x2
Definition: OperandsAndPartials.hpp:40
lgamma.hpp
stan::math::check_not_nan
bool check_not_nan(const char *function, const char *name, const T_y &y)
Return true if y is not NaN.
Definition: check_not_nan.hpp:63
check_positive_finite.hpp
stan
Definition: log_sum_exp.hpp:8
stan::math::value_of
T value_of(const fvar< T > &v)
Return the value of the specified variable.
Definition: value_of.hpp:16
digamma.hpp
stan::math::log
fvar< T > log(const fvar< T > &x)
Definition: log.hpp:15
stan::math::OperandsAndPartials::value
T_return_type value(double value)
Returns a T_return_type with the value specified with the partial derivatves.
Definition: OperandsAndPartials.hpp:68
square.hpp
stan::length
size_t length(const std::vector< T > &x)
Definition: length.hpp:10
is_constant_struct.hpp
stan::math::grad_reg_inc_gamma
T grad_reg_inc_gamma(T a, T z, T g, T dig, T precision=1e-6)
Definition: grad_reg_inc_gamma.hpp:16
gamma_q.hpp
stan::math::scaled_inv_chi_square_ccdf_log
return_type< T_y, T_dof, T_scale >::type scaled_inv_chi_square_ccdf_log(const T_y &y, const T_dof &nu, const T_scale &s)
Definition: scaled_inv_chi_square_ccdf_log.hpp:33
include_summand.hpp
check_nonnegative.hpp
stan::is_constant_struct
Metaprogram to determine if a type has a base scalar type that can be assigned to type double...
Definition: is_constant_struct.hpp:13
stan::math::exp
fvar< T > exp(const fvar< T > &x)
Definition: exp.hpp:10
stan::math::OperandsAndPartials
This class builds partial derivatives with respect to a set of operands.
Definition: OperandsAndPartials.hpp:38
VectorView.hpp
stan::math::OperandsAndPartials::d_x3
VectorView< T_return_type, false, true > d_x3
Definition: OperandsAndPartials.hpp:41
stan::max_size
size_t max_size(const T1 &x1, const T2 &x2)
Definition: max_size.hpp:9
VectorBuilder.hpp
partials_return_type.hpp
OperandsAndPartials.hpp
length.hpp
grad_reg_inc_gamma.hpp
stan::VectorBuilder
VectorBuilder allocates type T1 values to be used as intermediate values.
Definition: VectorBuilder.hpp:28
constants.hpp
stan::math::check_consistent_sizes
bool check_consistent_sizes(const char *function, const char *name1, const T1 &x1, const char *name2, const T2 &x2)
Return true if the dimension of x1 is consistent with x2.
Definition: check_consistent_sizes.hpp:31
stan::math::pow
fvar< T > pow(const fvar< T > &x1, const fvar< T > &x2)
Definition: pow.hpp:18
stan::math::check_nonnegative
bool check_nonnegative(const char *function, const char *name, const T_y &y)
Return true if y is non-negative.
Definition: check_nonnegative.hpp:66
value_of.hpp
stan::math::tgamma
fvar< T > tgamma(const fvar< T > &x)
Definition: tgamma.hpp:15
stan::VectorView
VectorView is a template expression that is constructed with a container or scalar, which it then allows to be used as an array using operator[].
Definition: VectorView.hpp:48
stan::partials_return_type::type
boost::math::tools::promote_args< typename partials_type< typename scalar_type< T1 >::type >::type, typename partials_type< typename scalar_type< T2 >::type >::type, typename partials_type< typename scalar_type< T3 >::type >::type, typename partials_type< typename scalar_type< T4 >::type >::type, typename partials_type< typename scalar_type< T5 >::type >::type, typename partials_type< typename scalar_type< T6 >::type >::type >::type type
Definition: partials_return_type.hpp:26
stan::math::gamma_q
fvar< T > gamma_q(const fvar< T > &x1, const fvar< T > &x2)
Definition: gamma_q.hpp:15
check_consistent_sizes.hpp
stan::math::check_positive_finite
bool check_positive_finite(const char *function, const char *name, const T_y &y)
Return true if y is positive and finite.
Definition: check_positive_finite.hpp:28
stan::math::OperandsAndPartials::d_x1
VectorView< T_return_type, false, true > d_x1
Definition: OperandsAndPartials.hpp:39
stan::math::negative_infinity
double negative_infinity()
Return negative infinity.
Definition: constants.hpp:132
stan::math::digamma
fvar< T > digamma(const fvar< T > &x)
Definition: digamma.hpp:16

[ Stan Home Page ] © 2011–2016, Stan Development Team.